@@ -8,14 +8,14 @@ import pyinotify
 wm = pyinotify.WatchManager()  # Watch Manager
 mask = pyinotify.IN_DELETE | pyinotify.IN_CREATE  # watched events
 
-class HandleEvents(pyinotify.ProcessEvent):
+class EventHandler(pyinotify.ProcessEvent):
     def process_IN_CREATE(self, event):
         print "Creating:", event.pathname
 
     def process_IN_DELETE(self, event):
         print "Removing:", event.pathname
 
-notifier = pyinotify.AsyncNotifier(wm, HandleEvents())
+notifier = pyinotify.AsyncNotifier(wm, EventHandler())
 wdd = wm.add_watch('/tmp', mask, rec=True)
 
 asyncore.loop()
