@@ -95,7 +95,7 @@ except ImportError:
 
 __author__ = "seb@dbzteam.org (Sebastien Martini)"
 
-__version__ = "0.8.9"
+__version__ = "0.9.0"
 
 __metaclass__ = type  # Use new-style classes by default
 
@@ -107,7 +107,17 @@ COMPATIBILITY_MODE = False
 
 
 # Load libc
-LIBC = ctypes.cdll.LoadLibrary(ctypes.util.find_library('c'))
+LIBC = None
+STRERRNO = None
+if sys.version_info[0] >= 2 and sys.version_info[1] >= 6:
+    LIBC = ctypes.CDLL(ctypes.util.find_library('c'), use_errno=True)
+    def _strerrno():
+        code = ctypes.get_errno()
+        return ' Errno=%s (%s)' % (os.strerror(code), errno.errorcode[code])
+    STRERRNO = _strerrno
+else:
+    LIBC = ctypes.CDLL(ctypes.util.find_library('c'))
+    STRERRNO = lambda : ''
 
 # The libc version > 2.4 check.
 # XXX: Maybe it is better to check if the libc has the needed functions inside?
@@ -445,11 +455,19 @@ class _RawEvent(_Event):
         @type name: string or None
         """
         # name: remove trailing '\0'
-        super(_RawEvent, self).__init__({'wd': wd,
-                                         'mask': mask,
-                                         'cookie': cookie,
-                                         'name': name.rstrip('\0')})
+        _Event.__init__(self, {'wd': wd,
+                               'mask': mask,
+                               'cookie': cookie,
+                               'name': name.rstrip('\0')})
         log.debug(repr(self))
+        # Use this variable to cache the result of str(self)
+        self._str = None
+
+    def __str__(self):
+        if self._str is None:
+            self._str = '%s %s %s %s' % (str(self.wd), str(self.mask),
+                                         str(self.cookie), self.name)
+        return self._str
 
 
 class Event(_Event):
@@ -602,22 +620,24 @@ class _SysProcessEvent(_ProcessEvent):
                 addw = self._watch_manager.add_watch
                 # The newly monitored directory inherits attributes from its
                 # parent directory.
-                newwd = addw(created_dir, watch_.mask, proc_fun=watch_.proc_fun,
-                             rec=False, auto_add=watch_.auto_add,
-                             exclude_filter=watch_.exclude_filter)
+                addw_ret = addw(created_dir, watch_.mask,
+                                proc_fun=watch_.proc_fun,
+                                rec=False, auto_add=watch_.auto_add,
+                                exclude_filter=watch_.exclude_filter)
 
                 # Trick to handle mkdir -p /t1/t2/t3 where t1 is watched and
                 # t2 and t3 are created.
                 # Since the directory is new, then everything inside it
                 # must also be new.
-                if newwd[created_dir] > 0:
+                created_dir_wd = addw_ret.get(created_dir)
+                if (created_dir_wd is not None) and created_dir_wd > 0:
                     for name in os.listdir(created_dir):
                         inner = os.path.join(created_dir, name)
                         if (os.path.isdir(inner) and
                             self._watch_manager.get_wd(inner) is None):
                             # Generate (simulate) creation event for sub
                             # directories.
-                            rawevent = _RawEvent(newwd[created_dir],
+                            rawevent = _RawEvent(created_dir_wd,
                                                  IN_CREATE | IN_ISDIR,
                                                  0, name)
                             self._notifier.append_event(rawevent)
@@ -681,16 +701,17 @@ class _SysProcessEvent(_ProcessEvent):
         if mv_:
             dest_path = mv_[0]
             watch_.path = dest_path
+            # add the separator to the source path to avoid overlapping
+            # path issue when testing with startswith()
+            src_path += os.path.sep
             src_path_len = len(src_path)
-            sep_len = len(os.path.sep)
             # The next loop renames all watches with src_path as base path.
             # It seems that IN_MOVE_SELF does not provide IN_ISDIR information
             # therefore the next loop is iterated even if raw_event is a file.
             for w in self._watch_manager.watches.values():
                 if w.path.startswith(src_path):
                     # Note that dest_path is a normalized path.
-                    w.path = os.path.join(dest_path,
-                                          w.path[src_path_len + sep_len:])
+                    w.path = os.path.join(dest_path, w.path[src_path_len:])
         else:
             log.error("The pathname '%s' of this watch %s has probably changed "
                       "and couldn't be updated, so it cannot be trusted "
@@ -999,7 +1020,7 @@ class Notifier:
         @param read_freq: if read_freq == 0, events are read asap,
                           if read_freq is > 0, this thread sleeps
                           max(0, read_freq - timeout) seconds. But if
-                          timeout is None it can be different because
+                          timeout is None it may be different because
                           poll is blocking waiting for something to read.
         @type read_freq: int
         @param threshold: File descriptor will be read only if the accumulated
@@ -1035,6 +1056,10 @@ class Notifier:
         self._read_freq = read_freq
         self._threshold = threshold
         self._timeout = timeout
+        # Coalesce events option
+        self._coalesce = False
+        # set of str(raw_event), only used when coalesce option is True
+        self._eventset = set()
 
     def append_event(self, event):
         """
@@ -1048,6 +1073,25 @@ class Notifier:
     def proc_fun(self):
         return self._default_proc_fun
 
+    def coalesce_events(self, coalesce=True):
+        """
+        Coalescing events. Events are usually processed by batchs, their size
+        depend on various factors. Thus, before processing them, events received
+        from inotify are aggregated in a fifo queue. If this coalescing
+        option is enabled events are filtered based on their unicity, only
+        unique events are enqueued, doublons are discarded. An event is unique
+        when the combination of its fields (wd, mask, cookie, name) is unique
+        among events of a same batch. After a batch of events is processed any
+        events is accepted again. By default this option is disabled, you have
+        to explictly call this function to turn it on.
+
+        @param coalesce: Optional new coalescing value. True by default.
+        @type coalesce: Bool
+        """
+        self._coalesce = coalesce
+        if not coalesce:
+            self._eventset.clear()
+
     def check_events(self, timeout=None):
         """
         Check for new events available to read, blocks up to timeout
@@ -1109,7 +1153,15 @@ class Notifier:
             # Retrieve name
             fname, = struct.unpack('%ds' % fname_len,
                                    r[rsum + s_size:rsum + s_size + fname_len])
-            self._eventq.append(_RawEvent(wd, mask, cookie, fname))
+            rawevent = _RawEvent(wd, mask, cookie, fname)
+            if self._coalesce:
+                # Only enqueue new (unique) events.
+                raweventstr = str(rawevent)
+                if raweventstr not in self._eventset:
+                    self._eventset.add(raweventstr)
+                    self._eventq.append(rawevent)
+            else:
+                self._eventq.append(rawevent)
             rsum += s_size + fname_len
 
     def process_events(self):
@@ -1121,13 +1173,21 @@ class Notifier:
         while self._eventq:
             raw_event = self._eventq.popleft()  # pop next event
             watch_ = self._watch_manager.get_watch(raw_event.wd)
+            if watch_ is None:
+                # Not really sure how we ended up here, nor how we should
+                # handle these types of events and if it is appropriate to
+                # completly skip them (like we are doing here).
+                log.warning("Unable to retrieve Watch object associated to %s",
+                            repr(raw_event))
+                continue
             revent = self._sys_proc_fun(raw_event)  # system processings
             if watch_ and watch_.proc_fun:
                 watch_.proc_fun(revent)  # user processings
             else:
                 self._default_proc_fun(revent)
         self._sys_proc_fun.cleanup()  # remove olds MOVED_* events records
-
+        if self._coalesce:
+            self._eventset.clear()
 
     def __daemonize(self, pid_file=None, force_kill=False, stdin=os.devnull,
                     stdout=os.devnull, stderr=os.devnull):
@@ -1217,12 +1277,17 @@ class Notifier:
 
     def loop(self, callback=None, daemonize=False, **args):
         """
-        Events are read only once time every min(read_freq, timeout)
+        Events are read only one time every min(read_freq, timeout)
         seconds at best and only if the size to read is >= threshold.
+        After this method returns it must not be called again for the same
+        instance.
 
-        @param callback: Functor called after each event processing. Expects
-                         to receive notifier object (self) as first parameter.
-        @type callback: callable
+        @param callback: Functor called after each event processing iteration.
+                         Expects to receive the notifier object (self) as first
+                         parameter. If this function returns True the loop is
+                         immediately terminated otherwise the loop method keeps
+                         looping.
+        @type callback: callable object or function
         @param daemonize: This thread is daemonized if set to True.
         @type daemonize: boolean
         @param args: Optional and relevant only if daemonize is True. Remaining
@@ -1237,24 +1302,26 @@ class Notifier:
         while 1:
             try:
                 self.process_events()
-                if callback is not None:
-                    callback(self)
+                if (callback is not None) and (callback(self) is True):
+                    break
                 ref_time = time.time()
                 # check_events is blocking
                 if self.check_events():
                     self._sleep(ref_time)
                     self.read_events()
             except KeyboardInterrupt:
-                # Unless sigint is caught (Control-C)
+                # Stop monitoring if sigint is caught (Control-C).
                 log.debug('Pyinotify stops monitoring.')
-                # Stop monitoring
-                self.stop()
                 break
+        # Close internals
+        self.stop()
+
 
     def stop(self):
         """
         Close inotify's instance (close its file descriptor).
         It destroys all existing watches, pending events,...
+        This method is automatically called at the end of loop().
         """
         self._pollobj.unregister(self._fd)
         os.close(self._fd)
@@ -1439,10 +1506,9 @@ class ExcludeFilter:
           ^/etc/rc.*
           ^/etc/hostname
 
-        @param arg_lst: is either a list or dict of patterns:
-                        [pattern1, ..., patternn] or a filename from which
+        @param arg_lst: is either a list of patterns or a filename from which
                         patterns will be loaded.
-        @type arg_lst: list(str) or str
+        @type arg_lst: list of str or str
         """
         if isinstance(arg_lst, str):
             lst = self._load_patterns_from_file(arg_lst)
@@ -1527,7 +1593,8 @@ class WatchManager:
         self._wmd = {}  # watch dict key: watch descriptor, value: watch
         self._fd = LIBC.inotify_init() # inotify's init, file descriptor
         if self._fd < 0:
-            raise OSError()
+            err = 'Cannot initialize new instance of inotify%s' % STRERRNO()
+            raise OSError(err)
 
     def get_fd(self):
         """
@@ -1569,10 +1636,9 @@ class WatchManager:
         """
         return self._wmd
 
-    def __add_watch(self, path, mask, proc_fun, auto_add, exclude_filter):
+    def __format_path(self, path):
         """
-        Add a watch on path, build a Watch object and insert it in the
-        watch manager dictionary. Return the wd value.
+        Format path to its internal (stored in watch manager) representation.
         """
         # Unicode strings are converted to byte strings, it seems to be
         # required because LIBC.inotify_add_watch does not work well when
@@ -1580,18 +1646,22 @@ class WatchManager:
         # Therefore even wd are indexed with bytes string and not with
         # unicode paths.
         if isinstance(path, unicode):
-            byte_path = path.encode(sys.getfilesystemencoding())
-        else:
-            byte_path = path
+            path = path.encode(sys.getfilesystemencoding())
+        return os.path.normpath(path)
 
+    def __add_watch(self, path, mask, proc_fun, auto_add, exclude_filter):
+        """
+        Add a watch on path, build a Watch object and insert it in the
+        watch manager dictionary. Return the wd value.
+        """
+        byte_path = self.__format_path(path)
         wd_ = LIBC.inotify_add_watch(self._fd,
                                      ctypes.create_string_buffer(byte_path),
                                      mask)
         if wd_ < 0:
             return wd_
-        watch_ = Watch(wd=wd_, path=os.path.normpath(byte_path), mask=mask,
-                       proc_fun=proc_fun, auto_add=auto_add,
-                       exclude_filter=exclude_filter)
+        watch_ = Watch(wd=wd_, path=byte_path, mask=mask, proc_fun=proc_fun,
+                       auto_add=auto_add, exclude_filter=exclude_filter)
         self._wmd[wd_] = watch_
         log.debug('New %s', watch_)
         return wd_
@@ -1613,6 +1683,9 @@ class WatchManager:
         although unicode paths are accepted there are converted to byte
         strings before a watch is put on that path. The encoding used for
         converting the unicode object is given by sys.getfilesystemencoding().
+        If |path| si already watched it is ignored, but if it is called with
+        option rec=True a watch is put on each one of its not-watched
+        subdirectory.
 
         @param path: Path to watch, the path can either be a file or a
                      directory. Also accepts a sequence (list) of paths.
@@ -1644,7 +1717,8 @@ class WatchManager:
         @return: dict of paths associated to watch descriptors. A wd value
                  is positive if the watch was added sucessfully,
                  otherwise the value is negative. If the path was invalid
-                 it is not included into this returned dictionary.
+                 or was already watched it is not included into this returned
+                 dictionary.
         @rtype: dict of {str: int}
         """
         ret_ = {} # return {path: wd, ...}
@@ -1658,14 +1732,19 @@ class WatchManager:
             for apath in self.__glob(npath, do_glob):
                 # recursively list subdirs according to rec param
                 for rpath in self.__walk_rec(apath, rec):
+                    if self.get_wd(rpath) is not None:
+                        # We decide to ignore paths already inserted into
+                        # the watch manager. Need to be removed with rm_watch()
+                        # first. Or simply call update_watch() to update it.
+                        continue
                     if not exclude_filter(rpath):
                         wd = ret_[rpath] = self.__add_watch(rpath, mask,
                                                             proc_fun,
                                                             auto_add,
                                                             exclude_filter)
                         if wd < 0:
-                            err = 'add_watch: cannot watch %s (WD=%d)'
-                            err = err % (rpath, wd)
+                            err = 'add_watch: cannot watch %s WD=%d%s'
+                            err = err % (rpath, wd, STRERRNO())
                             if quiet:
                                 log.error(err)
                             else:
@@ -1688,7 +1767,7 @@ class WatchManager:
         """
         for d in lpath:
             root = self.get_path(d)
-            if root:
+            if root is not None:
                 # always keep root
                 yield d
             else:
@@ -1760,8 +1839,8 @@ class WatchManager:
                 wd_ = addw(self._fd, ctypes.create_string_buffer(apath), mask)
                 if wd_ < 0:
                     ret_[awd] = False
-                    err = 'update_watch: cannot update WD=%d (%s)' % (wd_,
-                                                                      apath)
+                    err = 'update_watch: cannot update %s WD=%d%s'
+                    err = err % (apath, wd_, STRERRNO())
                     if quiet:
                         log.error(err)
                         continue
@@ -1776,7 +1855,7 @@ class WatchManager:
                 watch_.proc_fun = proc_fun
 
             if auto_add:
-                watch_.proc_fun = auto_add
+                watch_.auto_add = auto_add
 
             ret_[awd] = True
             log.debug('Updated watch - %s', self._wmd[awd])
@@ -1806,11 +1885,10 @@ class WatchManager:
         @return: WD or None.
         @rtype: int or None
         """
-        path = os.path.normpath(path)
+        path = self.__format_path(path)
         for iwd in self._wmd.items():
             if iwd[1].path == path:
                 return iwd[0]
-        log.debug('get_wd: unknown path %s', path)
 
     def get_path(self, wd):
         """
@@ -1822,9 +1900,8 @@ class WatchManager:
         @rtype: string or None
         """
         watch_ = self._wmd.get(wd)
-        if watch_:
+        if watch_ is not None:
             return watch_.path
-        log.debug('get_path: unknown WD %d', wd)
 
     def __walk_rec(self, top, rec):
         """
@@ -1872,7 +1949,7 @@ class WatchManager:
             wd_ = LIBC.inotify_rm_watch(self._fd, awd)
             if wd_ < 0:
                 ret_[awd] = False
-                err = 'rm_watch: cannot remove WD=%d' % awd
+                err = 'rm_watch: cannot remove WD=%d%s' % (awd, STRERRNO())
                 if quiet:
                     log.error(err)
                     continue
